// Datagram.cpp
//
// Copyright (C) 2011 Mike McCauley
// $Id: Datagram.cpp,v 1.6 2014/05/23 02:20:17 mikem Exp $

#include <Datagram.h>

Datagram::Datagram(GenericDriver& driver, uint8_t thisAddress) 
    :
    _driver(driver),
    _thisAddress(thisAddress)
{
}

////////////////////////////////////////////////////////////////////
// Public methods
bool Datagram::init()
{
    bool ret = _driver.init();
    if (ret)
	setThisAddress(_thisAddress);
    return ret;
}

void Datagram::setThisAddress(uint8_t thisAddress)
{
    _driver.setThisAddress(thisAddress);
    // Use this address in the transmitted FROM header
    setHeaderFrom(thisAddress);
    _thisAddress = thisAddress;
}

bool Datagram::sendto(uint8_t* buf, uint8_t len, uint8_t address)
{
    setHeaderTo(address);
    return _driver.send(buf, len);
}

bool Datagram::recvfrom(uint8_t* buf, uint8_t* len, uint8_t* from, uint8_t* to, uint8_t* id, uint8_t* flags)
{
    if (_driver.recv(buf, len))
    {
	if (from)  *from =  headerFrom();
	if (to)    *to =    headerTo();
	if (id)    *id =    headerId();
	if (flags) *flags = headerFlags();
	return true;
    }
    return false;
}

bool Datagram::available()
{
    return _driver.available();
}

void Datagram::waitAvailable()
{
    _driver.waitAvailable();
}

bool Datagram::waitPacketSent()
{
    return _driver.waitPacketSent();
}

bool Datagram::waitPacketSent(uint16_t timeout)
{
    return _driver.waitPacketSent(timeout);
}

bool Datagram::waitAvailableTimeout(uint16_t timeout)
{
    return _driver.waitAvailableTimeout(timeout);
}

uint8_t Datagram::thisAddress()
{
    return _thisAddress;
}

void Datagram::setHeaderTo(uint8_t to)
{
    _driver.setHeaderTo(to);
}

void Datagram::setHeaderFrom(uint8_t from)
{
    _driver.setHeaderFrom(from);
}

void Datagram::setHeaderId(uint8_t id)
{
    _driver.setHeaderId(id);
}

void Datagram::setHeaderFlags(uint8_t set, uint8_t clear)
{
    _driver.setHeaderFlags(set, clear);
}

uint8_t Datagram::headerTo()
{
    return _driver.headerTo();
}

uint8_t Datagram::headerFrom()
{
    return _driver.headerFrom();
}

uint8_t Datagram::headerId()
{
    return _driver.headerId();
}

uint8_t Datagram::headerFlags()
{
    return _driver.headerFlags();
}



